@@ -1,6 +1,3 @@
-#! /usr/bin/env python3
-
-
 # import dependencies
 import logging
 import random
@@ -8,7 +5,6 @@ from string import ascii_letters, digits, ascii_lowercase
 import subprocess
 import os
 from pathlib import Path
-import shutil
 from datetime import datetime
 import re
 import zc.lockfile
@@ -133,9 +129,17 @@ def lock(LOCKFILE):
 
 
 #____________________________________
-def unlock(lock, LOCKFILE):
+def unlock(lock, LOCKFILE, do_exit=True):
     lock.close()
     os.remove(LOCKFILE)
+    if do_exit:
+        exit()
+
+
+#____________________________________
+def unlock_if_false(function_return, lock, LOCKFILE):
+    if function_return == False:
+        unlock(lock, LOCKFILE)
 
 
 #____________________________________
@@ -210,25 +214,26 @@ def end_volume_setup_procedure(SUCCESS_FILE):
 
 
 #____________________________________
-def load_default_config():
-    if os.path.isfile('./defaults.conf'):
+def load_default_config(defaults_file='./defaults.conf'):
+    global cipher_algorithm, keysize, hash_algorithm, device, cryptdev, mountpoint, filesystem, paranoid, non_interactive, foreground, luks_cryptdev_file, luks_header_backup
+
+    if os.path.isfile(defaults_file):
         logging.info('Loading default configuration from defaults.conf')
-        import defaults.conf
-    else:
-        logging.info('No defaults.conf file found. Loading built-in variables.')
-        global cipher_algorithm, keysize, hash_algorithm, device, cryptdev, mountpoint, filesystem, paranoid, non_interactive, foreground, luks_cryptdev_file, luks_header_backup
-        cipher_algorithm='aes-xts-plain64'
-        keysize=256
-        hash_algorithm='sha256'
-        device='/dev/vdb'
-        cryptdev='crypt'
-        mountpoint='/export'
-        filesystem='ext4'
-        paranoid=False
-        non_interactive=False
-        foreground=False
-        luks_cryptdev_file='/tmp/luks-cryptdev.ini'
-        luks_header_backup='/tmp/luks-header.bck'
+        config = ConfigParser()
+        config.read_file(open(defaults_file))
+        defaults = config['defaults']
+        cipher_algorithm = defaults['cipher_algorithm']
+        keysize = defaults['keysize']
+        hash_algorithm = defaults['hash_algorithm']
+        device = defaults['device']
+        cryptdev = defaults['cryptdev']
+        mountpoint = defaults['mountpoint']
+        filesystem = defaults['filesystem']
+        paranoid = defaults['paranoid']
+        non_interactive = defaults['non_interactive']
+        foreground = defaults['foreground']
+        luks_cryptdev_file = defaults['luks_cryptdev_file']
+        luks_header_backup = defaults['luks_header_backup']
 
 
 #____________________________________
@@ -314,21 +319,18 @@ def write_secret_to_vault(vault_url, wrapping_token, secret_path, user_key, user
 
 #____________________________________
 # Used in setup_device
-def check_passphrase(passphrase, passphrase_confirmation, passphrase_length):
+def check_passphrase(passphrase_length, passphrase, passphrase_confirmation):
     if passphrase_length == None:
         if passphrase == None:
             echo('ERROR', "Missing passphrase!")
-            #unlock(lock, LOCKFILE)
             return False
         if passphrase_confirmation == None:
             echo('ERROR', 'Missing confirmation passphrase!')
-            #unlock(lock, LOCKFILE)
             return False
         if passphrase == passphrase_confirmation:
             s3cret = passphrase
         else:
             echo('ERROR', 'No matching passphrases!')
-            #unlock(lock, LOCKFILE)
             return False
     else:
             s3cret = create_random_secret(passphrase_length)
@@ -363,7 +365,7 @@ class device:
                 logging.error('Device not mounted, exiting! Please check logfile:')
                 logging.error(f'No device mounted to {self.mountpoint}')
                 run_command('df -h', log_stderr_stdout=True)
-                # TODO: unlock and terminate process
+                return False # TODO: unlock and terminate process
     
     def is_encrypted(self):
         logging.debug('Checking if the volume is already encrypted.')
@@ -388,9 +390,9 @@ class device:
     def luksOpen(self, s3cret):
         return run_command(f'printf "{s3cret}\n" | cryptsetup luksOpen {self.device_name} {self.cryptdev}')
 
-    def setup_device(self, vault_url, wrapping_token, secret_path, user_key, luks_header_backup_dir,
-                    luks_header_backup_file, lock, LOCKFILE, cipher_algorithm, keysize, hash_algorithm,
-                    passphrase=None, passphrase_confirmation=None, passphrase_length=None):
+    def setup_device(self, luks_header_backup_dir, luks_header_backup_file, cipher_algorithm, keysize, hash_algorithm,
+                    # vault_url, wrapping_token, secret_path, user_key
+                    passphrase_length, passphrase, passphrase_confirmation):
             echo('INFO', 'Start the encryption procedure.')
             logging.info(f'Using {cipher_algorithm} algorithm to luksformat the volume.')
             logging.debug('Start cryptsetup')
@@ -398,19 +400,20 @@ class device:
             logging.debug('Cryptsetup full command:')
             logging.debug('cryptsetup -v --cipher $cipher_algorithm --key-size $keysize --hash $hash_algorithm --iter-time 2000 --use-urandom --verify-passphrase luksFormat $device --batch-mode')
 
-            s3cret = check_passphrase(passphrase, passphrase_confirmation, passphrase_length)
+            s3cret = check_passphrase(passphrase_length, passphrase, passphrase_confirmation)
             if s3cret == False:
                 return False # TODO: unlock and exit
             
             # Start encryption procedure
-            luksFormat(self, s3cret, cipher_algorithm, keysize, hash_algorithm)
+            self.luksFormat(s3cret, cipher_algorithm, keysize, hash_algorithm)
 
             # Write the secret to vault
-            write_secret_to_vault(vault_url, wrapping_token, secret_path, user_key, s3cret)
+            # write_secret_to_vault(vault_url, wrapping_token, secret_path, user_key, s3cret)
 
             # Backup LUKS header
-            os.mkdir(luks_header_backup_dir)
-            _, _, luksHeaderBackup_ec = luksHeaderBackup(self, luks_header_backup_dir, luks_header_backup_file)
+            if not os.path.isdir(luks_header_backup_dir):
+                os.mkdir(luks_header_backup_dir)
+            _, _, luksHeaderBackup_ec = self.luksHeaderBackup(luks_header_backup_dir, luks_header_backup_file)
 
             if luksHeaderBackup_ec != 0:
                 # Cryptsetup returns 0 on success and a non-zero value on error.
@@ -423,30 +426,32 @@ class device:
                 logging.error(f'Command cryptsetup failed with exit code {luksHeaderBackup_ec}! Mounting {self.device_name} to {self.mountpoint} and exiting.')
                 if luksHeaderBackup_ec == 2:
                     echo('ERROR', 'Bad passphrase. Please try again.')
-                # TODO: unlock and exit
+                return False # TODO: unlock and exit
+
+            return s3cret
     
     def open_device(self, s3cret):
         echo('INFO', 'Open LUKS volume')
         if not Path(f'/dev/mapper{self.cryptdev}').is_block_device():
-            _, _, openec = luksOpen(self, s3cret)
+            _, _, openec = self.luksOpen(s3cret)
             
             if openec != 0:
                 if openec == 2:
                     echo('ERROR', 'Bad passphrase. Please try again.')
-                    # TODO: unlock and exit
+                    return False # TODO: unlock and exit
                 else:
                     echo('ERROR', f'Crypt device already exists! Please check logs: {LOGFILE}')
                     logging.error('Unable to luksOpen device.')
                     logging.error(f'/dev/mapper/{self.cryptdev} already exists.')
                     logging.error(f'Mounting {self.device_name} to {self.mountpoint} again.')
                     run_command(f'mount {self.device_name} {self.mountpoint}', log_stderr_stdout=True)
-                    # TODO: unlock and exit
+                    return False # TODO: unlock and exit
     
     def encryption_status(self):
         logging.info(f'Check {self.cryptdev} status with cryptsetup status')
         run_command(f'cryptsetup -v status {self.cryptdev}', log_stderr_stdout=True)
     
-    def create_cryptdev_ini_file(self, now, cipher_algorithm, has_algorithm, keysize, luksUUID, luks_header_backup_dir, luks_header_backup_file):
+    def create_cryptdev_ini_file(self, luks_cryptdev_file, cipher_algorithm, hash_algorithm, keysize, luks_header_backup_dir, luks_header_backup_file, now=now):
         luksUUID, _, _ = run_command(f'cryptsetup luksUUID {self.device_name}')
 
         with open(luks_cryptdev_file, 'w') as f:
@@ -490,10 +495,81 @@ class device:
         if mkfs_ec != 0:
             echo('ERROR', f'While creating {self.filesystem} filesystem. Please check logs.')
             echo('ERROR', 'Command mkfs failed!')
-            # TODO: unlock and exit
+            return False #TODO: unlock and exit
     
-    def mount_volume(self):
+    def mount_vol(self):
         echo('INFO', 'Mounting encrypted device.')
         logging.info(f'Mounting /dev/mapper/{self.cryptdev} to {self.mountpoint}')
         run_command(f'mount /dev/mapper/{self.cryptdev} {self.mountpoint}', log_stderr_stdout=True)
         run_command('df -Hv', log_stderr_stdout=True)
+
+    def encrypt(self, cipher_algorithm, keysize, hash_algorithm, luks_header_backup_dir, luks_header_backup_file, 
+               LOCKFILE, SUCCESS_FILE, luks_cryptdev_file, # vault_url, wrapping_token, secret_path, user_key,
+               passphrase_length, passphrase, passphrase_confirmation):
+        
+        locked = lock(LOCKFILE) # Create lock file
+
+        cryptdev = create_random_cryptdev_name() # Assign random name to cryptdev
+
+        check_cryptsetup() # Check that cryptsetup and dmsetup are installed
+
+        unlock_if_false(self.check_vol(), locked, LOCKFILE) # Check which virtual volume is mounted to mountpoint, unlock and exit if it's not mounted
+
+        if not self.is_encrypted(): # Check if the volume is encrypted, if it's not start the encryption procedure
+            self.umount_vol()
+            s3cret = self.setup_device(luks_header_backup_dir, luks_header_backup_file, cipher_algorithm, keysize, hash_algorithm,
+                                        # vault_url, wrapping_token, secret_path, user_key,
+                                        passphrase_length, passphrase, passphrase_confirmation)
+            unlock_if_false(s3cret, locked, LOCKFILE)
+
+            with open('./s3cret_file','w') as sf: # TODO: !!!!!! REMOVE THIS AFTER TROUBLESHOOTING !!!!!!
+                sf.write(s3cret)
+        
+        unlock_if_false(self.open_device(s3cret), locked, LOCKFILE) # Create mapping
+
+        self.encryption_status() # Check status
+
+        self.create_cryptdev_ini_file(luks_cryptdev_file, cipher_algorithm, hash_algorithm, keysize, luks_header_backup_dir, luks_header_backup_file) # Create ini file
+
+        end_encrypt_procedure(SUCCESS_FILE) # LUKS encryption finished. Print end dialogue.
+
+        unlock(locked, LOCKFILE, do_exit=False) # Unlock
+    
+    def volume_setup(self, cipher_algorithm, hash_algorithm, keysize, luksUUID, luks_header_backup_dir,
+                     luks_header_backup_file, LOCKFILE, SUCCESS_FILE):
+        
+        locked = lock(LOCKFILE) # Create lock file
+
+        unlock_if_false(self.create_fs(), locked, LOCKFILE) # Create filesystem
+
+        self.mount_vol() # Mount volume
+
+        self.create_cryptdev_ini_file(now, cipher_algorithm, hash_algorithm, keysize, luksUUID,
+                                      luks_header_backup_dir, luks_header_backup_file) # Update ini file
+        
+        end_volume_setup_procedure(SUCCESS_FILE) # Volume setup finished. Print end dialogue
+
+        unlock(locked, LOCKFILE, do_exit=False) # Unlock once done
+
+
+def main_script(device_name='/dev/vdb', cryptdev='crypt', mountpoint='/export', filesystem='ext4',
+                cipher_algorithm='aes-xts-plain64', keysize=256, hash_algorithm='sha256', luks_header_backup_dir='/etc/luks',
+                luks_header_backup_file='luks-header.bck', luks_cryptdev_file='/etc/luks/luks-cryptdev.ini',
+                passphrase_length=8, passphrase=None, passphrase_confirmation=None):
+    
+    device = fastluks.device(device_name, cryptdev, mountpoint, filesystem)
+    
+    LOCKFILE = '/var/run/fast-luks-encryption.lock'
+    SUCCESS_FILE = '/var/run/fast-luks-encryption.success'
+    
+    device.encrypt(cipher_algorithm, keysize, hash_algorithm, luks_header_backup_dir, luks_header_backup_file, 
+                   LOCKFILE, SUCCESS_FILE, luks_cryptdev_file, passphrase_length, passphrase, passphrase_confirmation)
+                   # vault_url, wrapping_token, secret_path, user_key
+
+    variables = fastluks.read_ini_file(luks_cryptdev_file)
+    luksUUID = variables['uuid']
+    LOCKFILE = '/var/run/fast-luks-volume-setup.lock'
+    SUCCESS_FILE = '/var/run/fast-luks-volume-setup.success'
+
+    device.volume_setup(cipher_algorithm, hash_algorithm, keysize, luksUUID, luks_header_backup_dir,
+                        luks_header_backup_file, LOCKFILE, SUCCESS_FILE)
\ No newline at end of file
